componere
=========
*composing classes at runtime since 2018*

[![Build Status](https://travis-ci.org/krakjoe/componere.svg?branch=master)](https://travis-ci.org/krakjoe/componere)

This extension allows the composition and re-composition of classes at runtime.

This extension also allows monkey patching an instance with a runtime composed class.

All of this is done using dark scary magic, but of the kind you might take to production.

API
===

  * InvalidArgumentException is thrown when a method or constructor is called incorrectly
  * RuntimeException is thrown when the call was invalid because of the environment that made it
  * When a method returns the parent type, it is suitable for chaining
  * All classes are final, and do not support properties
  * Only Componere\Method supports cloning

```php
namespace Componere {
	namespace Abstract {
		final abstract class Definition {
			public function addTrait(string $name) : Definition;
			public function addInterface(string $name) : Definition;
			public function addMethod(string $name, Method $method) : Definition;
		}
	}

	final class Definition extends Abstract\Definition {
		public function __construct(string $name);
		public function __construct(string $name, string $parent);
		public function __construct(string $name, array $interfaces);
		public function __construct(string $name, string $parent, array $interfaces);

		public function addProperty(string $name, Value $property) : Definition;
		public function addConstant(string $name, Value $constant) : Definition; 

		public function getClosure(string $name) : Closure;
		public function getClosures() : array;

		public function register();
		public function isRegistered() : bool;
	}
	final class Patch extends Abstract\Definition {
		public function __construct(object $instance);
		public function __construct(object $instance, array $interfaces);

		public function getClosure(string $name) : Closure;
		public function getClosures() : array;

		public function apply();
		public function revert();
		public function isApplied() : bool;
	}
	final class Method {
		public function __construct(string $name, Closure $method);

		public function setProtected() : Method;
		public function setPrivate() : Method;
		public function setStatic() : Method;
	}
	final class Value {
		public function __construct($value);

		public function setProtected() : Value;
		public function setPrivate() : Value;
		public function setStatic() : Value;
	}
}
```

Explanation
===========

These are for the weak ...
